@@ -8,6 +8,8 @@
 
 -- |
 -- An abstract interface for maps from JSON keys to values.
+--
+-- @since 2.0.0.0
 
 module Data.Aeson.KeyMap (
     -- * Map Type
@@ -16,6 +18,7 @@ module Data.Aeson.KeyMap (
     -- * Query
     null,
     lookup,
+    (!?),
     size,
     member,
 
@@ -25,6 +28,7 @@ module Data.Aeson.KeyMap (
 
     -- ** Insertion
     insert,
+    insertWith,
 
     -- * Deletion
     delete,
@@ -65,6 +69,7 @@ module Data.Aeson.KeyMap (
     -- * Traversal
     -- ** Map
     map,
+    mapWithKey,
     mapKeyVal,
     traverse,
     traverseWithKey,
@@ -85,6 +90,9 @@ module Data.Aeson.KeyMap (
     filterWithKey,
     mapMaybe,
     mapMaybeWithKey,
+
+    -- * Key Type
+    Key,
 ) where
 
 -- Import stuff from Prelude explicitly
@@ -162,18 +170,7 @@ delete k (KeyMap m) = KeyMap (M.delete k m)
 
 -- | 'alterF' can be used to insert, delete, or update a value in a map.
 alterF :: Functor f => (Maybe v -> f (Maybe v)) -> Key -> KeyMap v -> f (KeyMap v)
-#if MIN_VERSION_containers(0,5,8)
 alterF f k = fmap KeyMap . M.alterF f k . unKeyMap
-#else
-alterF f k m = fmap g (f mv) where
-    g r =  case r of
-        Nothing -> case mv of
-            Nothing -> m
-            Just _  -> delete k m
-        Just v' -> insert k v' m
-
-    mv = lookup k m
-#endif
 
 -- | Return the value to which the specified key is mapped,
 -- or Nothing if this map contains no mapping for the key.
@@ -186,11 +183,19 @@ lookup t tm = M.lookup t (unKeyMap tm)
 insert :: Key -> v -> KeyMap v -> KeyMap v
 insert k v tm = KeyMap (M.insert k v (unKeyMap tm))
 
+-- | Insert with a function combining new and old values, taken in that order.
+--
+-- @since 2.1.1.0
+insertWith :: (a -> a -> a) -> Key -> a -> KeyMap a -> KeyMap a
+insertWith f k v m = KeyMap (M.insertWith f k v (unKeyMap m))
+
 -- | Map a function over all values in the map.
 map :: (a -> b) -> KeyMap a -> KeyMap b
 map = fmap
 
 -- | Map a function over all values in the map.
+--
+-- @since 2.1.0.0
 mapWithKey :: (Key -> a -> b) -> KeyMap a -> KeyMap b
 mapWithKey f (KeyMap m) = KeyMap (M.mapWithKey f m)
 
@@ -234,6 +239,10 @@ fromListWith op = KeyMap . M.fromListWith op
 -- |  Construct a map with the supplied mappings. If the
 -- list contains duplicate mappings, the later mappings take
 -- precedence.
+--
+-- >>> fromList [("a", 'x'), ("a", 'y')]
+-- fromList [("a",'y')]
+--
 fromList :: [(Key, v)] -> KeyMap v
 fromList = KeyMap . M.fromList
 
@@ -245,7 +254,7 @@ toList = M.toList . unKeyMap
 
 -- | Return a list of this map' elements.
 --
--- @since 2.0.2.0
+-- @since 2.0.3.0
 elems :: KeyMap v -> [v]
 elems = M.elems . unKeyMap
 
@@ -259,7 +268,7 @@ toAscList = M.toAscList . unKeyMap
 difference :: KeyMap v -> KeyMap v' -> KeyMap v
 difference tm1 tm2 = KeyMap (M.difference (unKeyMap tm1) (unKeyMap tm2))
 
--- The (left-biased) union of two maps. It prefers the first map when duplicate
+-- | The (left-biased) union of two maps. It prefers the first map when duplicate
 -- keys are encountered, i.e. ('union' == 'unionWith' 'const').
 union :: KeyMap v -> KeyMap v -> KeyMap v
 union (KeyMap x) (KeyMap y) = KeyMap (M.union x y)
@@ -387,11 +396,19 @@ lookup t tm = H.lookup t (unKeyMap tm)
 insert :: Key -> v -> KeyMap v -> KeyMap v
 insert k v tm = KeyMap (H.insert k v (unKeyMap tm))
 
+-- | Insert with a function combining new and old values, taken in that order.
+--
+-- @since 2.1.1.0
+insertWith :: (a -> a -> a) -> Key -> a -> KeyMap a -> KeyMap a
+insertWith f k v m = KeyMap (H.insertWith f k v (unKeyMap m))
+
 -- | Map a function over all values in the map.
 map :: (a -> b) -> KeyMap a -> KeyMap b
 map = fmap
 
 -- | Map a function over all values in the map.
+--
+-- @since 2.1.0.0
 mapWithKey :: (Key -> a -> b) -> KeyMap a -> KeyMap b
 mapWithKey f (KeyMap m) = KeyMap (H.mapWithKey f m)
 
@@ -446,7 +463,7 @@ toList = H.toList . unKeyMap
 
 -- | Return a list of this map' elements.
 --
--- @since 2.0.2.0
+-- @since 2.0.3.0
 elems :: KeyMap v -> [v]
 elems = H.elems . unKeyMap
 
@@ -460,7 +477,7 @@ toAscList = sortBy (comparing fst) . toList
 difference :: KeyMap v -> KeyMap v' -> KeyMap v
 difference tm1 tm2 = KeyMap (H.difference (unKeyMap tm1) (unKeyMap tm2))
 
--- The (left-biased) union of two maps. It prefers the first map when duplicate
+-- | The (left-biased) union of two maps. It prefers the first map when duplicate
 -- keys are encountered, i.e. ('union' == 'unionWith' 'const').
 union :: KeyMap v -> KeyMap v -> KeyMap v
 union (KeyMap x) (KeyMap y) = KeyMap (H.union x y)
@@ -541,6 +558,16 @@ mapMaybeWithKey f (KeyMap m) = KeyMap (H.mapMaybeWithKey f m)
 -- combinators using existing abstractions
 -------------------------------------------------------------------------------
 
+-- | Return the value to which the specified key is mapped,
+-- or Nothing if this map contains no mapping for the key.
+--
+-- This is a flipped version of 'lookup'.
+--
+-- @since 2.1.1.0
+--
+(!?) :: KeyMap v -> Key -> Maybe v
+(!?) m k = lookup k m
+
 -- | Generalized union with combining function.
 alignWith :: (These a b -> c) -> KeyMap a -> KeyMap b -> KeyMap c
 alignWith f (KeyMap x) (KeyMap y) = KeyMap (SA.alignWith f x y)
