@@ -1,4 +1,3 @@
-{-# LANGUAGE NoImplicitPrelude #-}
 -- |
 -- Module:      Data.Aeson.Types
 -- Copyright:   (c) 2011-2016 Bryan O'Sullivan
@@ -51,6 +50,7 @@ module Data.Aeson.Types
     -- ** Encoding
     , ToJSON(..)
     , KeyValue(..)
+    , KeyValueOmit(..)
 
     -- ** Keys for maps
     , ToJSONKey(..)
@@ -73,14 +73,18 @@ module Data.Aeson.Types
     -- ** Liftings to unary and binary type constructors
     , FromJSON1(..)
     , parseJSON1
+    , omittedField1
     , FromJSON2(..)
     , parseJSON2
+    , omittedField2
     , ToJSON1(..)
     , toJSON1
     , toEncoding1
+    , omitField1
     , ToJSON2(..)
     , toJSON2
     , toEncoding2
+    , omitField2
 
     -- ** Generic JSON classes
     , GFromJSON
@@ -112,13 +116,19 @@ module Data.Aeson.Types
     , (.:?)
     , (.:!)
     , (.!=)
+    , (.:?=)
+    , (.:!=)
     , object
     , parseField
     , parseFieldMaybe
     , parseFieldMaybe'
+    , parseFieldOmit
+    , parseFieldOmit'
     , explicitParseField
     , explicitParseFieldMaybe
     , explicitParseFieldMaybe'
+    , explicitParseFieldOmit
+    , explicitParseFieldOmit'
 
     , listEncoding
     , listValue
@@ -133,6 +143,7 @@ module Data.Aeson.Types
     , constructorTagModifier
     , allNullaryToStringTag
     , omitNothingFields
+    , allowOmittedFields
     , sumEncoding
     , unwrapUnaryRecords
     , tagSingleConstructors
@@ -150,6 +161,9 @@ module Data.Aeson.Types
     , keyModifier
     , defaultJSONKeyOptions
 
+    -- * Parsing exceptions
+    , AesonException (..)
+
     -- * Parsing context
     , (<?>)
     , JSONPath
@@ -159,8 +173,6 @@ module Data.Aeson.Types
     , formatError
     ) where
 
-import Prelude.Compat
-
 import Data.Aeson.Encoding (Encoding, unsafeToEncoding, fromEncoding, Series, pairs)
 import Data.Aeson.Types.Class
 import Data.Aeson.Types.Internal
